<?php 

/**
 * 
 * Auth API 
 *
 * This file provides custom API endpoints for the booking desks plugin. It includes an authentication endpoint to
 * generate JWT tokens based on provided credentials, and a revoke endpoint to invalidate JWT tokens, essentially
 * logging out users on the client-side. The plugin uses the firebase/php-jwt library for JWT token encoding and decoding.
 * 
 */


// Security: Prevent direct access to the file
defined( 'ABSPATH' ) or die( 'No script kiddies please!' );


// Define your secret key here
define('YOUR_SECRET_KEY', 'bd29c84c-2fa8-4472-aee2-aa936e3089c0');

// Authentication endpoint
function booking_desks_auth_endpoint() {
    $credentials = json_decode(file_get_contents('php://input'), true);

    // Check if login, password, and secret key are provided
    if (empty($credentials['username']) || empty($credentials['password']) || empty($credentials['secret_key'])) {
        return rest_ensure_response(array('error' => 'Invalid request. Please provide username, password, and secret_key.'), 400);
    }

    // Perform your custom authentication logic here
    $username = sanitize_text_field($credentials['username']);
    $password = $credentials['password'];
    $provided_secret_key = $credentials['secret_key'];

    // Check if the provided secret key matches the predefined secret key
    if ($provided_secret_key !== YOUR_SECRET_KEY) {
        return rest_ensure_response(array('error' => 'Unauthorized. Invalid secret key.'), 401);
    }

    // Replace the following with your own authentication logic
    $user = wp_authenticate($username, $password);
    if (is_wp_error($user)) {
        return rest_ensure_response(array('error' => 'Invalid credentials.'), 401);
    }

    $issuer = get_home_url();
    // Generate a JWT token
    $token_payload = array(
        'iss' => $issuer,
        'exp' => time() + (14 * 24 * 60 * 60),
        'isa' => time(),
        'data' =>[
            'user_id' => $user->ID,
        ] 
    );

    // Sign the token with the secret key and specify the algorithm (HS256 in this case)
    $token = Firebase\JWT\JWT::encode($token_payload, YOUR_SECRET_KEY, 'HS256');

    return rest_ensure_response(array('token' =>'Bearer '. $token), 200);
}

add_action('rest_api_init', function () {
    register_rest_route('booking-desks/v1', '/auth', array(
        'methods' => 'POST',
        'callback' => 'booking_desks_auth_endpoint',
    ));
});



// Function to validate JWT token
function validate_jwt_token($token, $secret_key)
{
    global $wpdb;
    $table_name = $wpdb->prefix . 'token_revoked';

    // Check if the token exists in the 'token_revoked' table
    $existing_token = $wpdb->get_var($wpdb->prepare("SELECT token FROM $table_name WHERE token = %s", $token));
    if ($existing_token) {
        return array('valid' => false, 'error' => 'Token has expired.');
    }

    try {
        $token = explode(' ', $token)[1];
        $decoded_token = Firebase\JWT\JWT::decode($token, new Firebase\JWT\Key($secret_key, 'HS256'));
        return array('valid' => true, 'data' => (array) $decoded_token, 'token' => $token);
    } catch (Exception $e) {
        return array('valid' => false, 'error' => $e->getMessage());
    }
}

// Function to validate API key
function validate_api_key($headers)
{
    if (!isset($headers['X-API-KEY'])) {
        return false;
    }

    $api_key = $headers['X-API-KEY'];
    // Replace 'your-api-key-here' with the actual API key you want to use
    $valid_api_key = YOUR_SECRET_KEY;

    return $api_key === $valid_api_key;
}


// Function to create 'token_revoked' table if it doesn't exist
function create_token_revoked_table()
{
    global $wpdb;
    $table_name = $wpdb->prefix . 'token_revoked';
    $charset_collate = $wpdb->get_charset_collate();

    if ($wpdb->get_var("SHOW TABLES LIKE '$table_name'") != $table_name) {
        $sql = "CREATE TABLE $table_name (
            token VARCHAR(255) NOT NULL,
            timestamp DATETIME NOT NULL,
            PRIMARY KEY (token)
        ) $charset_collate;";
        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        dbDelta($sql);
    }
}

// Revoke token endpoint
function booking_desks_revoke_token_endpoint()
{
    // Get the JWT token from the request headers
    $headers = getallheaders();
    if (!validate_api_key($headers)) {
        return rest_ensure_response(array('error' => 'Invalid API key.'), 401);
    }

    if (!isset($headers['Authorization'])) {
        return rest_ensure_response(array('error' => 'Missing Authorization header.'), 400);
    }

    $token = $headers['Authorization'];
    $secret_key = YOUR_SECRET_KEY;

    // Validate the JWT token
    $validation_result = validate_jwt_token($token, $secret_key);
    if (!$validation_result['valid']) {
        return rest_ensure_response(array('message' => 'Token has been revoked.'), 200);
    }

    // Create the 'token_revoked' table if it doesn't exist
    create_token_revoked_table();

    global $wpdb;
    $table_name = $wpdb->prefix . 'token_revoked';

    // Insert the token and timestamp into the 'token_revoked' table
    $timestamp = current_time('mysql');
    $wpdb->insert($table_name, array('token' => $validation_result['token'], 'timestamp' => $timestamp));

    return rest_ensure_response(array('message' => 'Token has been revoked.'), 200);
}

add_action('rest_api_init', function () {
    register_rest_route('booking-desks/v1', '/revoke-token', array(
        'methods' => 'POST',
        'callback' => 'booking_desks_revoke_token_endpoint',
    ));
});


// Add a custom user role for API access
function custom_add_api_access_role() {
    add_role( 'api_access', 'API Access', array(
        'read' => true,
    ));
}
add_action( 'init', 'custom_add_api_access_role' );



function st_create_api_user() {
    $username = 'api_user'; 
    $password = 'HMB3q1R9V17$'; 
    $email = 'api_user@innovarooms.project-progress.online';


    $existing_user = get_user_by( 'login', $username );


    if ( ! $existing_user ) {
        $user_id = wp_create_user( $username, $password, $email );

        if ( ! is_wp_error( $user_id ) ) {
            $user = get_user_by( 'ID', $user_id );

            $user->set_role( 'api_access' );
        }
    
    }
}
add_action( 'init', 'st_create_api_user' );



